@@ -132,9 +132,29 @@ def big_tnl(gis):
     logging.info(updates)
 
 
+def test_add_with_dates(gis):
+
+    #: Get the first feature
+    df_with_datetimes = pd.DataFrame.spatial.from_featureclass(
+        r'C:\gis\Projects\FastData\FastData.gdb\roads_datetime_test_3857'
+    )
+    itemid = '9fa86322135f44feb476c2e2c01566fc'
+    first = df_with_datetimes.iloc[:1].copy()
+
+    #: Convert two date fields to string and pd.datetime
+    first['effective'] = first['effective'].astype('string')
+    first['updated'] = pd.to_datetime(first['updated'])
+    # first.spatial.project(3857)
+
+    updates = palletjack.load.FeatureServiceUpdater.add_features(gis, itemid, first)
+    logging.info('number of features loaded:')
+    logging.info(updates)
+    #: TODO: For some reason, we're not getting the geometry on this feature.
+
+
 gis = arcgis.gis.GIS(username='Jake.Adams@UtahAGRC')
 
-test_adds(gis)
+# test_adds(gis)
 
 # test_updates_no_geometry(gis)
 # test_updates_geometry(gis)
@@ -145,6 +165,8 @@ test_adds(gis)
 
 # big_tnl(gis)
 
+test_add_with_dates(gis)
+
 #: TODO: size is working at 10mb size limit (100 and 50 didn't work) but now I'm getting {"layerName":"slco_lir_20221229","submissionTime":1672793254217,"lastUpdatedTime":1672793264930,"recordCount":0,"status":"Failed","error":{"code":500,"description":"Input string was not in a correct format.Couldn't store \u003c\u003e in parcel_acres Column.  Expected type is Double."}} errors (huh?)
 
 # def ceil_div(num, denom):
