@@ -21,8 +21,10 @@ class FeatureServiceInlineUpdater:
     """Updates an AGOL Feature Service with data from a pandas DataFrame
     """
 
-    def __init__(self, gis, dataframe, index_column):
+    def __init__(self, gis, dataframe, index_column, field_mapping=None):
         self.gis = gis
+        if field_mapping:
+            dataframe = utils.rename_fields(dataframe, field_mapping)
         self.new_dataframe = dataframe.rename(columns=utils.rename_columns_for_agol(dataframe.columns))
         self.index_column = utils.rename_columns_for_agol([index_column])[index_column]
         try:
@@ -249,11 +251,13 @@ class FeatureServiceInlineUpdater:
             dict: Messages returned from append operation
         """
 
-        geojson = dataframe.spatial.to_featureset().to_geojson
+        feature_set = utils.fix_numeric_empty_strings(
+            dataframe.spatial.to_featureset(), target_featurelayer.properties.fields
+        )
         result, messages = utils.retry(
             target_featurelayer.append,
             upload_format='geojson',
-            edits=geojson,
+            edits=feature_set.to_geojson,
             upsert=True,
             return_messages=True,
             rollback=True,
@@ -609,6 +613,7 @@ class FeatureServiceOverwriter:
             target_featurelayer.append,
             upload_format='geojson',
             edits=geojson,
+            #:TODO figure out a way to preserve GUIDs?
             upsert=False,
             return_messages=True,
             rollback=True,
