@@ -1,6 +1,8 @@
 import logging
 from pathlib import Path
 
+import arcgis
+import geopandas as gpd
 import numpy as np
 import pandas as pd
 import pytest
@@ -254,6 +256,40 @@ class TestGoogleDriveDownloader:
         ] == [rec.message for rec in caplog.records]
         assert result is None
 
+    def test_download_file_from_google_drive_pauses_appropriately(self, mocker):
+        sleep_mock = mocker.patch.object(palletjack.loaders, 'sleep')
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_file_id_from_sharing_link')
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_http_response', return_value='response')
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_filename_from_response', return_value='baz.png')
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_save_response_content')
+
+        downloader = palletjack.GoogleDriveDownloader('/foo/bar')
+
+        downloader.download_file_from_google_drive('1234', 42, 1.5)
+
+        sleep_mock.assert_called_with(1.5)
+        sleep_mock.assert_called_once()
+
+    def test_download_attachments_from_dataframe_sleeps_specified_time(self, mocker):
+
+        sleep_mock = mocker.patch.object(palletjack.loaders, 'sleep')
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_file_id_from_sharing_link')
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_http_response', return_value='response')
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_filename_from_response', return_value='baz.png')
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_save_response_content')
+
+        downloader = palletjack.GoogleDriveDownloader('/foo/bar')
+
+        sheet_dataframe = pd.DataFrame({
+            'join_id': [1],
+            'link': ['a'],
+        })
+
+        downloader.download_attachments_from_dataframe(sheet_dataframe, 'link', 'join_id', 'path')
+
+        sleep_mock.assert_called_with(5)
+        sleep_mock.assert_called_once()
+
     def test_download_attachments_from_dataframe_handles_multiple_rows(self, mocker):
 
         downloader_mock = mocker.Mock()
@@ -312,6 +348,178 @@ class TestGoogleDriveDownloader:
             file_id = downloader._get_file_id_from_sharing_link(test_link)
 
 
+class TestGoogleDriveDownloaderAPI:
+
+    def test_get_request_and_filename_from_drive_api_returns_both(self, mocker):
+        client = mocker.Mock()
+        client.drive.service.files.return_value.get_media.return_value = 'request'
+        client.drive.service.files.return_value.get.return_value.execute.return_value = {'name': 'foo.bar'}
+
+        request, filename = palletjack.GoogleDriveDownloader._get_request_and_filename_from_drive_api(
+            mocker.Mock(), client, 'bar'
+        )
+
+        assert request == 'request'
+        assert filename == 'foo.bar'
+
+    def test_get_request_and_filename_from_drive_api_adds_extension(self, mocker):
+        client = mocker.Mock()
+        client.drive.service.files.return_value.get_media.return_value = 'request'
+        client.drive.service.files.return_value.get.return_value.execute.return_value = {
+            'name': 'foo',
+            'mimeType': 'image/jpeg'
+        }
+
+        request, filename = palletjack.GoogleDriveDownloader._get_request_and_filename_from_drive_api(
+            mocker.Mock(), client, 'bar'
+        )
+
+        assert request == 'request'
+        assert filename == 'foo.jpg'
+
+    def test_get_request_and_filename_from_drive_api_warns_on_missing_mimeType(self, mocker, caplog):
+        client = mocker.Mock()
+        client.drive.service.files.return_value.get_media.return_value = 'request'
+        client.drive.service.files.return_value.get.return_value.execute.return_value = {
+            'name': 'foo',
+        }
+
+        downloader = palletjack.GoogleDriveDownloader('/foo/bar')
+
+        caplog.set_level(logging.DEBUG, logger='palletjack.loaders.GoogleDriveDownloader')
+        caplog.clear()
+
+        request, filename = downloader._get_request_and_filename_from_drive_api(client, 'bar')
+
+        assert ['bar: No MIME type in drive info, file extension not set'] == [rec.message for rec in caplog.records]
+        assert request == 'request'
+        assert filename == 'foo'
+
+    def test_get_request_and_filename_from_drive_api_warns_on_cant_find_extension(self, mocker, caplog):
+        client = mocker.Mock()
+        client.drive.service.files.return_value.get_media.return_value = 'request'
+        client.drive.service.files.return_value.get.return_value.execute.return_value = {
+            'name': 'foo',
+            'mimeType': 'bee/boo'
+        }
+
+        downloader = palletjack.GoogleDriveDownloader('/foo/bar')
+
+        caplog.set_level(logging.DEBUG, logger='palletjack.loaders.GoogleDriveDownloader')
+        caplog.clear()
+
+        request, filename = downloader._get_request_and_filename_from_drive_api(client, 'bar')
+
+        assert ['bar: Unable to determine file extension from MIME type, file extension not set'] == \
+            [rec.message for rec in caplog.records]
+        assert request == 'request'
+        assert filename == 'foo'
+
+    def test_save_get_media_content_works_multiple_chunks(self, mocker):
+        mocker.patch.object(palletjack.loaders, 'BytesIO')
+        downloader_mock = mocker.patch.object(palletjack.loaders, 'MediaIoBaseDownload')
+        downloader_mock.return_value.next_chunk.side_effect = [(1, False), (2, True)]
+
+        palletjack.GoogleDriveDownloader._save_get_media_content(mocker.Mock(), 'foo', mocker.Mock())
+
+        assert downloader_mock.return_value.next_chunk.call_count == 2
+
+    def test_download_file_from_google_drive_using_api_creates_filename(self, mocker):
+
+        mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_file_id_from_sharing_link')
+        mocker.patch.object(
+            palletjack.GoogleDriveDownloader,
+            '_get_request_and_filename_from_drive_api',
+            return_value=('response', 'baz.png')
+        )
+        save_mock = mocker.patch.object(palletjack.GoogleDriveDownloader, '_save_get_media_content')
+
+        downloader = palletjack.GoogleDriveDownloader('/foo/bar')
+
+        downloader.download_file_from_google_drive_using_api(mocker.Mock(), '1234', 42)
+
+        save_mock.assert_called_with('response', Path('/foo/bar/baz.png'))
+
+    def test_download_file_from_google_drive_using_api_handles_empty_string_link(self, mocker, caplog):
+
+        file_id_mock = mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_file_id_from_sharing_link')
+
+        downloader = palletjack.GoogleDriveDownloader('/foo/bar')
+
+        caplog.set_level(logging.DEBUG, logger='palletjack.loaders.GoogleDriveDownloader')
+        caplog.clear()
+        result = downloader.download_file_from_google_drive_using_api(mocker.Mock(), '', 42)
+
+        file_id_mock.assert_not_called()
+        assert ['Row 42 has no attachment info'] == [rec.message for rec in caplog.records]
+        assert result is None
+
+    def test_download_file_from_google_drive_using_api_handles_None_link(self, mocker, caplog):
+
+        file_id_mock = mocker.patch.object(palletjack.GoogleDriveDownloader, '_get_file_id_from_sharing_link')
+
+        downloader = palletjack.GoogleDriveDownloader('/foo/bar')
+
+        caplog.set_level(logging.DEBUG, logger='palletjack.loaders.GoogleDriveDownloader')
+        caplog.clear()
+        result = downloader.download_file_from_google_drive_using_api(mocker.Mock(), None, 42)
+
+        file_id_mock.assert_not_called()
+        assert ['Row 42 has no attachment info'] == [rec.message for rec in caplog.records]
+        assert result is None
+
+    def test_download_file_from_google_drive_using_api_handles_download_error(self, mocker, caplog):
+
+        mocker.patch.object(
+            palletjack.GoogleDriveDownloader, '_get_file_id_from_sharing_link', return_value='google_id'
+        )
+        mocker.patch.object(palletjack.utils, 'sleep')
+        mocker.patch.object(
+            palletjack.GoogleDriveDownloader, '_get_request_and_filename_from_drive_api', side_effect=Exception('Boom')
+        )
+
+        saver_mock = mocker.patch.object(palletjack.GoogleDriveDownloader, '_save_get_media_content')
+
+        downloader = palletjack.GoogleDriveDownloader('/foo/bar')
+
+        caplog.set_level(logging.DEBUG, logger='palletjack.loaders.GoogleDriveDownloader')
+        caplog.clear()
+        result = downloader.download_file_from_google_drive_using_api(mocker.Mock(), '1234', 42)
+
+        saver_mock.assert_not_called()
+        assert [
+            'Row 42: downloading file 1234',
+            'Row 42: extracted file id google_id',
+            'Row 42: Couldn\'t download 1234',
+            'Boom',
+        ] == [rec.message for rec in caplog.records]
+        assert result is None
+
+    def test_download_attachments_from_dataframe_using_api_handles_multiple_rows(self, mocker):
+
+        mocker.patch.object(palletjack.utils, 'authorize_pygsheets')
+
+        downloader_mock = mocker.Mock()
+        downloader_mock.download_file_from_google_drive_using_api.side_effect = ['foo/bar.png', 'baz/boo.png']
+
+        sheet_dataframe = pd.DataFrame({
+            'join_id': [1, 2],
+            'link': ['a', 'b'],
+        })
+
+        downloaded_df = palletjack.GoogleDriveDownloader.download_attachments_from_dataframe_using_api(
+            downloader_mock, 'service_file', sheet_dataframe, 'link', 'join_id', 'path'
+        )
+
+        test_df = pd.DataFrame({
+            'join_id': [1, 2],
+            'link': ['a', 'b'],
+            'path': ['foo/bar.png', 'baz/boo.png'],
+        })
+
+        tm.assert_frame_equal(downloaded_df, test_df)
+
+
 class TestSFTPLoader:
 
     def test_download_sftp_folder_contents_uses_right_credentials(self, mocker):
@@ -387,3 +595,17 @@ class TestSFTPLoader:
         palletjack.SFTPLoader.read_csv_into_dataframe(sftploader_mock, 'baz')
 
         pd_mock.assert_called_with(Path('foo', 'baz'), names=None, dtype=None)
+
+
+class TestPostgresLoader:
+
+    def test_get_postgres_connection(self, mocker):
+
+        mocker.patch.object(
+            gpd, 'read_postgis', return_value=gpd.read_file(gpd.datasets.get_path('naturalearth_lowres'))
+        )
+
+        loader = palletjack.PostgresLoader('host', 'app', 'user', 'password')
+        dataframe = loader.read_table_into_dataframe('table', 'name', '4326', 'geometry')
+
+        assert dataframe.spatial.geometry_type == ['polygon']
