@@ -1721,22 +1721,20 @@ class TestFeatureServiceOverwriter:
             0] == 'Failed to append data to layer id 0 in itemid abc. Append should have been rolled back.'
 
     def test_truncate_and_load_feature_service_normal(self, mocker):
-        mock_fl = mocker.Mock()
+        mock_fl = mocker.MagicMock()
         mock_fl.manager.truncate.return_value = {
             'submissionTime': 123,
             'lastUpdatedTime': 124,
             'status': 'Completed',
         }
-        mock_fl.properties = {
-            'fields': [
-                {
-                    'name': 'Foo'
-                },
-                {
-                    'name': 'Bar'
-                },
-            ]
-        }
+        mock_fl.properties.fields = [
+            {
+                'name': 'Foo'
+            },
+            {
+                'name': 'Bar'
+            },
+        ]
         mock_fl.append.return_value = (True, {'recordCount': 42})
 
         fl_class_mock = mocker.Mock()
@@ -1760,16 +1758,14 @@ class TestFeatureServiceOverwriter:
             'lastUpdatedTime': 124,
             'status': 'Completed',
         }
-        mock_fl.properties = {
-            'fields': [
-                {
-                    'name': 'Foo'
-                },
-                {
-                    'name': 'Bar'
-                },
-            ]
-        }
+        mock_fl.properties.fields = [
+            {
+                'name': 'Foo'
+            },
+            {
+                'name': 'Bar'
+            },
+        ]
         mock_fl.append.side_effect = [(False, 'foo'), (True, {'recordCount': 42})]
 
         fl_class_mock = mocker.Mock()
@@ -1798,16 +1794,14 @@ class TestFeatureServiceOverwriter:
             'lastUpdatedTime': 124,
             'status': 'Completed',
         }
-        mock_fl.properties = {
-            'fields': [
-                {
-                    'name': 'Foo'
-                },
-                {
-                    'name': 'Bar'
-                },
-            ]
-        }
+        mock_fl.properties.fields = [
+            {
+                'name': 'Foo'
+            },
+            {
+                'name': 'Bar'
+            },
+        ]
         mock_fl.append.return_value = (True, {'recordCount': 42})
 
         fl_class_mock = mocker.Mock()
@@ -1925,8 +1919,8 @@ class TestFeatureServiceOverwriter:
 class TestFeatureServiceInlineUpdaterUpsert:
 
     def test_upsert_new_data_doesnt_raise_on_normal(self, mocker):
-        mock_df = mocker.Mock()
-        mock_fl = mocker.Mock()
+        mock_df = mocker.MagicMock()
+        mock_fl = mocker.MagicMock()
         mock_fl.append.return_value = (True, {'message': 'foo'})
         mocker.patch('palletjack.utils.rename_columns_for_agol')
         updater = palletjack.FeatureServiceInlineUpdater(mocker.Mock(), mocker.Mock(), 'foo')
@@ -1934,8 +1928,8 @@ class TestFeatureServiceInlineUpdaterUpsert:
         updater._upsert_new_data(mock_fl, mock_df, 'abc', 0)
 
     def test_upsert_new_data_retries_on_httperror(self, mocker):
-        mock_df = mocker.Mock()
-        mock_fl = mocker.Mock()
+        mock_df = mocker.MagicMock()
+        mock_fl = mocker.MagicMock()
         mock_fl.append.side_effect = [Exception, (True, {'message': 'foo'})]
         mocker.patch('palletjack.utils.rename_columns_for_agol')
 
@@ -1943,8 +1937,8 @@ class TestFeatureServiceInlineUpdaterUpsert:
         updater._upsert_new_data(mock_fl, mock_df, 'abc', 0)
 
     def test_upsert_new_data_raises_on_False_result(self, mocker):
-        mock_df = mocker.Mock()
-        mock_fl = mocker.Mock()
+        mock_df = mocker.MagicMock()
+        mock_fl = mocker.MagicMock()
         mock_fl.append.return_value = (False, {'message': 'foo'})
         mocker.patch('palletjack.utils.rename_columns_for_agol')
 
@@ -1957,28 +1951,30 @@ class TestFeatureServiceInlineUpdaterUpsert:
             0] == 'Failed to append data to layer id 0 in itemid abc. Append should have been rolled back.'
 
     def test_upsert_new_data_in_hosted_feature_layer_normal(self, mocker):
-        mock_fl = mocker.Mock()
+        mock_fl = mocker.MagicMock()
         # mock_fl.manager.truncate.return_value = {
         #     'submissionTime': 123,
         #     'lastUpdatedTime': 124,
         #     'status': 'Completed',
         # }
-        mock_fl.properties = {
-            'fields': [
-                {
-                    'name': 'Foo'
-                },
-                {
-                    'name': 'Bar'
-                },
-            ],
-            'indexes': [
-                {
-                    'fields': 'Foo',
-                    'isUnique': True
-                },
-            ]
-        }
+        mock_fl.properties.fields = [
+            {
+                'name': 'Foo',
+                'type': 'esriFieldTypeString',
+                'nullable': True,
+            },
+            {
+                'name': 'Bar',
+                'type': 'esriFieldTypeString',
+                'nullable': True,
+            },
+        ]
+        mock_fl.properties.indexes = [
+            {
+                'fields': 'Foo',
+                'isUnique': True
+            },
+        ]
 
         mock_fl.append.return_value = (True, {'recordCount': 42})
 
@@ -1995,29 +1991,31 @@ class TestFeatureServiceInlineUpdaterUpsert:
 
         assert uploaded_features == 42
 
-    def test_upsert_new_data_in_hosted_feature_layer_handles_field_renaming(self, mocker):
-        mock_fl = mocker.Mock()
+    def test_upsert_new_data_in_hosted_feature_layer_handles_agol_field_renaming(self, mocker):
+        mock_fl = mocker.MagicMock()
         # mock_fl.manager.truncate.return_value = {
         #     'submissionTime': 123,
         #     'lastUpdatedTime': 124,
         #     'status': 'Completed',
         # }
-        mock_fl.properties = {
-            'fields': [
-                {
-                    'name': 'Foo_field'
-                },
-                {
-                    'name': 'Bar'
-                },
-            ],
-            'indexes': [
-                {
-                    'fields': 'Bar',
-                    'isUnique': True
-                },
-            ]
-        }
+        mock_fl.properties.fields = [
+            {
+                'name': 'Foo_field',
+                'type': 'esriFieldTypeString',
+                'nullable': True,
+            },
+            {
+                'name': 'Bar',
+                'type': 'esriFieldTypeString',
+                'nullable': True,
+            },
+        ]
+        mock_fl.properties.indexes = [
+            {
+                'fields': 'Bar',
+                'isUnique': True
+            },
+        ]
         mock_fl.append.return_value = (True, {'recordCount': 42})
 
         fl_class_mock = mocker.Mock()
@@ -2033,6 +2031,45 @@ class TestFeatureServiceInlineUpdaterUpsert:
 
         assert uploaded_features == 42
 
+    def test_upsert_new_data_in_hosted_feature_layer_handles_manual_field_renaming(self, mocker):
+        mock_fl = mocker.MagicMock()
+
+        mock_fl.properties.fields = [
+            {
+                'name': 'Shape__Length',
+                'type': 'esriFieldTypeString',
+                'nullable': True,
+            },
+            {
+                'name': 'Bar',
+                'type': 'esriFieldTypeString',
+                'nullable': True,
+            },
+        ]
+        mock_fl.properties.indexes = [
+            {
+                'fields': 'Bar',
+                'isUnique': True
+            },
+        ]
+        mock_fl.append.return_value = (True, {'recordCount': 42})
+
+        fl_class_mock = mocker.Mock()
+        fl_class_mock.fromitem.return_value = mock_fl
+        mocker.patch('arcgis.features.FeatureLayer', fl_class_mock)
+
+        new_dataframe = pd.DataFrame(columns=['st_length(shape)', 'Bar'])
+        field_mapping = {'st_length(shape)': 'Shape__Length'}
+        mocker.patch.object(pd.DataFrame, 'spatial')
+
+        updater = palletjack.FeatureServiceInlineUpdater(
+            mocker.Mock(), new_dataframe, 'Bar', field_mapping=field_mapping
+        )
+
+        uploaded_features = updater.upsert_new_data_in_hosted_feature_layer('abc')
+
+        assert uploaded_features == 42
+
 
 class TestFeatureServiceInlineUpdaterInit:
 
